@@ -1,5 +1,5 @@
 """
-DocWriter is a tool for writing documents in ASCII, HTML, 
+DocWriter is a tool for writing documents in ASCII, HTML,
 LaTeX, Doconce, and other formats based on input from Python
 datastructures.
 
@@ -16,8 +16,8 @@ import re, os, glob, commands
 class _BaseWriter:
     """
     Base class for document writing classes.
-    Each subclass implements a specific format (HTML, LaTeX,
-    reStructuredText, etc.).
+    Each subclass implements a specific format (html, latex,
+    rst, etc.).
     """
     def __init__(self, format, filename_extension):
         # use StringIO as a string "file" for writing the document:
@@ -25,7 +25,7 @@ class _BaseWriter:
         self.filename_extension = filename_extension
         self.format = format
         self._footer_called = False
-        
+
     document = property(fget=lambda self: self.file.getvalue(),
                         doc='Formatted document as a string')
 
@@ -40,7 +40,7 @@ class _BaseWriter:
         if not self._footer_called:
             self.footer()
             self._footer_called = True
-            
+
         f = open(filename + self.filename_extension, 'w')
         f.write(self.document)
         f.close()
@@ -52,16 +52,16 @@ class _BaseWriter:
     def header(self):
         """Header as required by format. Called in constructor."""
         pass
-    
+
     def footer(self):
         """Footer as required by format. Called in write_to_file."""
         pass
-    
+
     def not_impl(self, method):
         raise NotImplementedError, \
               'method "%s" in class "%s" is not implemented' % \
               (method, self.__class__.__name__)
-    
+
     def title(self, title, authors_and_institutions=[], date='today'):
         """
         Provide title and authors.
@@ -80,7 +80,7 @@ class _BaseWriter:
         """Return a string with today's date suitably formatted."""
         import time
         return time.strftime('%a, %d %b %Y (%H:%M)')
-    
+
     def section(self, title, label=None):
         """
         Write a section heading with the given title and an optional
@@ -149,7 +149,7 @@ class _BaseWriter:
             if tag_replacement is not None:
                 text = c.sub(tag_replacement, text)
         return text
-    
+
     def list(self, items, listtype='itemize'):
         """
         Write list or nested lists.
@@ -160,7 +160,7 @@ class _BaseWriter:
         # call _BaseWriter.unfold_list to traverse the list
         # and use self.item_handler to typeset each item
         self.not_impl('list')
-        
+
     def unfold_list(self, items, item_handler, listtype, level=0):
         """
         Traverse a possibly nested list and call item_handler for
@@ -207,7 +207,7 @@ class _BaseWriter:
         @param keyword: the keyword of the item in a 'description' list.
         """
         self.not_impl('item_handler')
-                    
+
     def verbatim(self, code):
         """
         Write verbatim text in fixed-width form
@@ -261,7 +261,7 @@ class _BaseWriter:
         if failure:
             print 'Could not convert;\n  %s' % cmd
         return final
-        
+
     def figure(self, filename, caption, width=None, height=None, label=None):
         """
         Insert a figure into the document.
@@ -324,7 +324,7 @@ INLINE_TAGS = {
     'math':
     r'%s\$(?P<subst>[^ `][^$`]*)\$%s' % \
     (inline_tag_begin, inline_tag_end),
-    
+
     # $latex text$|$pure text alternative$
     'math2':
     r'%s\$(?P<latexmath>[^ `][^$`]*)\$\|\$(?P<puretext>[^ `][^$`]*)\$%s' % \
@@ -334,12 +334,12 @@ INLINE_TAGS = {
     'emphasize':
     r'%s\*(?P<subst>[^ `][^*`]*)\*%s' % \
     (inline_tag_begin, inline_tag_end),
-    
+
     # `verbatim inline text is enclosed in back quotes`
     'verbatim':
     r'%s`(?P<subst>[^ ][^`]*)`%s' % \
     (inline_tag_begin, inline_tag_end),
-    
+
     # _underscore before and after signifies bold_
     'bold':
     r'%s_(?P<subst>[^ `][^_`]*)_%s' % \
@@ -349,7 +349,7 @@ INLINE_TAGS = {
 class Doconce(_BaseWriter):
     def __init__(self):
         _BaseWriter.__init__(self, 'Doconce', '.do.txt')
-    
+
     def title(self, title, authors_and_institutions=[], date='today'):
         s = '\nTITLE: %s\n' % title
         for ai in authors_and_institutions:
@@ -388,7 +388,7 @@ class Doconce(_BaseWriter):
         #text = _BaseWriter.expandtext(self, text,
         #                              INLINE_TAGS, HTML.INLINE_TAGS_SUBST)
         self.file.write(text)
-        
+
     def list(self, items, listtype='itemize'):
         self.unfold_list(items, self.item_handler, listtype)
 
@@ -411,7 +411,7 @@ class Doconce(_BaseWriter):
                 elif listtype == 'description':
                     s += '\n%s%s- %s: %s' % (indent, indent, keyword, item)
         self.file.write(s)
-                    
+
     def verbatim(self, code):
         self.file.write('\n!bc\n' + r'%s' % code + '\n!ec\n')
 
@@ -441,7 +441,7 @@ class Doconce(_BaseWriter):
             s += '   | ' + ' | '.join(row) + ' |\n'
         s += '   |' + '-'*(width-2) + '|\n\n'
         self.file.write(s)
-    
+
     def url(self, url_address, link_text=None):
         if link_text is None:
             link_text = 'link'  # problems with Doconce and empty link text
@@ -466,7 +466,7 @@ class HTML(_BaseWriter):
     table_border = '2'
     table_cellpadding = '5'
     table_cellspacing = '2'
-    
+
     INLINE_TAGS_SUBST = {  # from inline tags to HTML tags
         # keep math as is:
         'math': None,  # indicates no substitution
@@ -475,52 +475,52 @@ class HTML(_BaseWriter):
         'bold':          r'\g<begin><b>\g<subst></b>\g<end>',
         'verbatim':      r'\g<begin><tt>\g<subst></tt>\g<end>',
         }
-    
+
     def __init__(self):
-        _BaseWriter.__init__(self, 'HTML', '.html')
+        _BaseWriter.__init__(self, 'html', '.html')
         self.header()
 
     def header(self):
         s = """\
 <!-- HTML document generated by %s.%s -->
-<HTML>
-<BODY BGCOLOR="white">
+<html>
+<body bgcolor="white">
 """ % (__name__, self.__class__.__name__)
         self.file.write(s)
-    
+
     def footer(self):
         s = """
-</BODY>
-</HTML>
+</body>
+</html>
 """
         self.file.write(s)
-    
+
     def title(self, title, authors_and_institutions=[], date='today'):
         s = """
-<TITLE>%s</TITLE>
-<CENTER><H1>%s</H1></CENTER>
+<title>%s</title>
+<center><h1>%s</h1></center>
 """ % (title, title)
         for ai in authors_and_institutions:
             author = ai[0]
             s += """
-<CENTER>
-<H4>%s</H4>""" % author
+<center>
+<h4>%s</h4>""" % author
             for inst in ai[1:]:
                 s += """
-<H6>%s</H6>""" % inst
-            s += """\n</CENTER>\n\n"""
+<h6>%s</h6>""" % inst
+            s += """\n</center>\n\n"""
         if date is not None:
             if date == 'today':
                 date = self.today_date()
-            s += """<CENTER>%s</CENTER>\n\n\n""" % date
+            s += """<center>%s</center>\n\n\n""" % date
         self.file.write(s)
         self.paragraph_separator()
 
     def heading(self, level, title, label=None):
         if label is None:
-            s = """\n<H%d>%s</H%d>\n""" % (level, title, level)
+            s = """\n<h%d>%s</h%d>\n""" % (level, title, level)
         else:
-            s = """\n<H%d><A HREF="%s">%s</H%d>\n""" % \
+            s = """\n<h%d><a href="%s">%s</h%d>\n""" % \
                 (level, label, title, level)
         self.file.write(s)
 
@@ -534,21 +534,21 @@ class HTML(_BaseWriter):
         self.heading(4, title, label)
 
     def paragraph(self, title, ending='.', label=None):
-        s = '\n\n<P><!-- paragraph with heading -->\n<B>%s%s</B>\n' \
+        s = '\n\n<p><!-- paragraph with heading -->\n<b>%s%s</b>\n' \
             % (title, ending)
         if label is not None:
-            s += '<A NAME="%s">\n' % label
+            s += '<a name="%s">\n' % label
         self.file.write(s)
 
     def paragraph_separator(self):
-        self.file.write('\n<P>\n')
+        self.file.write('\n<p>\n')
 
     def text(self, text, indent=0):
         text = _BaseWriter.text(self, text, indent)
         text = _BaseWriter.expandtext(self, text,
                                       INLINE_TAGS, HTML.INLINE_TAGS_SUBST)
         self.file.write(text)
-        
+
     def list(self, items, listtype='itemize'):
         self.unfold_list(items, self.item_handler, listtype)
 
@@ -557,65 +557,65 @@ class HTML(_BaseWriter):
         s = ''
         if item == '_begin':
             if listtype == 'itemize':
-                s += '\n%s<UL>' % indent
+                s += '\n%s<ul>' % indent
             elif listtype == 'enumerate':
-                s += '\n%s<OL>' % indent
+                s += '\n%s<ol>' % indent
             elif listtype == 'description':
-                s += '\n%s<DL>' % indent
+                s += '\n%s<dl>' % indent
             s += ' <!-- start of "%s" list -->\n' % listtype
         elif item == '_end':
             if listtype == 'itemize':
-                s += '%s</UL>' % indent
+                s += '%s</ul>' % indent
             elif listtype == 'enumerate':
-                s += '%s</OL>' % indent
+                s += '%s</ol>' % indent
             elif listtype == 'description':
-                s += '%s</DL>' % indent
+                s += '%s</dl>' % indent
             s += ' <!-- end of "%s" list -->\n' % listtype
         else:
             # ordinary item:
             if item is not None:
                 if listtype in ('itemize', 'enumerate'):
-                    s += '%s%s<P><LI> %s\n' % (indent, indent, item)
+                    s += '%s%s<p><li> %s\n' % (indent, indent, item)
                 else:
-                    s += '%s%s<P><DT>%s</DT><DD>%s</DD>\n' % \
+                    s += '%s%s<p><dt>%s</dt><dd>%s</dd>\n' % \
                          (indent, indent, keyword, item)
         self.file.write(s)
-                    
+
     def verbatim(self, code):
-        self.file.write('\n<PRE>' + r'%s' % code + '\n</PRE>\n')
+        self.file.write('\n<pre>' + r'%s' % code + '\n</pre>\n')
 
     def figure(self, filename, caption, width=None, height=None, label=None):
         filename = self.figure_conversion(filename, ('.jpg', '.gif', '.png'))
         if width:
-            width = ' WIDTH=%s ' % width
+            width = ' width=%s ' % width
         else:
             width = ''
         if height:
-            height = ' WIDTH=%s ' % width
+            height = ' width=%s ' % width
         else:
             height = ''
-        s = '\n<HR><IMG SRC="%s"%s%s>\n<P><EM>%s</EM>\n<HR><P>\n' % \
+        s = '\n<hr><img src="%s"%s%s>\n<p><em>%s</em>\n<hr><p>\n' % \
             (filename, width, height, caption)
         self.file.write(s)
 
     def table(self, table, column_headline_pos='c', column_pos='c'):
-        s = '\n<P>\n<TABLE BORDER="%s" CELLPADDING="%s" CELLSPACING="%s">\n' %\
+        s = '\n<p>\n<table border="%s" cellpadding="%s" cellspacing="%s">\n' %\
             (HTML.table_border, HTML.table_cellpadding, HTML.table_cellspacing)
         for line in table:
-            s += '<TR>'
+            s += '<tr>'
             for column in line:
-                s += '<TD>%s</TD>' % column
-            s += '</TR>\n'
-        s += '</TABLE>\n\n'
+                s += '<td>%s</td>' % column
+            s += '</tr>\n'
+        s += '</table>\n\n'
         self.file.write(s)
-    
+
     def url(self, url_address, link_text=None):
         if link_text is None:
             link_text = url_address
-        self.file.write('\n<A HREF="%s">%s</A>\n' % (url_address, link_text))
+        self.file.write('\n<a href="%s">%s</a>\n' % (url_address, link_text))
 
     def link(self, link_text, link_target):
-        self.file.write('\n<A HREF="%s">%s</A>\n' % (link_text, link_target))
+        self.file.write('\n<a href="%s">%s</a>\n' % (link_text, link_target))
 
     # autogenerate doc strings by combining parent class doc strings
     # with subclass doc strings:
@@ -644,9 +644,9 @@ class DocWriter:
     methods = 'title', 'section', 'subsection', 'subsubsection', \
               'paragraph', 'paragraph_separator', 'text', 'list', \
               'verbatim', 'math', 'raw', 'url', 'link', \
-              'write_to_file', 'figure', 'table', 
-    
-        
+              'write_to_file', 'figure', 'table',
+
+
     def __init__(self, *formats):
         """
         @param formats: sequence of strings specifying the desired formats.
@@ -664,7 +664,7 @@ class DocWriter:
                   writer.__class__.__name__ + '*'*60
             s += str(writer)
         return s
-            
+
     def dispatcher(self, *args, **kwargs):
         #print 'in dispatcher for', self.method_name, 'with args', args, kwargs
         #self.history = (self.method_name, args, kwargs)
@@ -684,7 +684,7 @@ class DocWriter:
     # can use inspect module to extract doc of all methods and
     # put this doc in __doc__
     '''
-    
+
 # Autogenerate methods in class DocWriter (with right
 # method signature and doc strings stolen from class _BaseWriter (!)):
 
@@ -722,13 +722,15 @@ func_to_method(_%s, DocWriter, '%s')
 
 def html_movie(plotfiles, interval_ms=300, width=800, height=600,
                casename='movie'):
-    """ 
-    Takes a list plotfiles which should be for example of the form:
+    """
+    Takes a list plotfiles which should be for example of the form::
+
         ['frame00.png', 'frame01.png', ... ]
+
     where each string should be the name of an image file and they should be
     in the proper order for viewing as an animation.
 
-    The result is html text strings that incorporate javascript to 
+    The result is html text strings that incorporate javascript to
     loop through the plots one after another.  The html text also features
     buttons for controlling the movie.
     The parameter iterval_ms is the time interval between loading
@@ -743,9 +745,9 @@ def html_movie(plotfiles, interval_ms=300, width=800, height=600,
     viewed in a browser. The images variable in the javascript code
     is unique for each movie, because it is annotated by the casename
     string, so several such javascript sections can be used in the
-    same html file. 
+    same html file.
 
-    This function is based on code written by R.J. LeVeque, based on 
+    This function is based on code written by R.J. LeVeque, based on
     a template from Alan McIntyre.
     """
     import os
@@ -756,7 +758,7 @@ def html_movie(plotfiles, interval_ms=300, width=800, height=600,
     missing_files = [fname for fname in plotfiles if not os.path.isfile(fname)]
     if missing_files:
         raise ValueError('Missing plot files: %s' % str(missing_files)[1:-1])
-        
+
     ext = os.path.splitext(plotfiles[0])[-1]
     if ext == '.png' or ext == '.jpg' or ext == '.jpeg' or ext == 'gif':
         pass
@@ -776,7 +778,7 @@ def html_movie(plotfiles, interval_ms=300, width=800, height=600,
 var num_images_%(casename)s = %(no_images)d;
 var img_width = %(width)d;
 var img_height = %(height)d;
-var interval = %(interval_ms)d;    
+var interval = %(interval_ms)d;
 var images_%(casename)s = new Array();
 
 function preload_images_%(casename)s()
@@ -852,12 +854,12 @@ def html_movie_embed(moviefile, width=400, height=400):
     handling of such files.
     """
     text = """
-<EMBED src="%(moviefile)s"
-WIDTH="%(width)s"
-HEIGHT="%(height)s"
-AUTOPLAY="TRUE"
-LOOP="TRUE">
-</EMBED>
+<embed src="%(moviefile)s"
+width="%(width)s"
+height="%(height)s"
+autoplay="false"
+loop="true">
+</embed>
 """ % vars()
     return text
 
@@ -893,12 +895,12 @@ showstatusbar="true">
 -->
 <!--
 Attributes of the <embed> tag are:
-SRC - tells what file to use.
-AUTOSTART="TRUE" - tells the computer to start the Video playing upon loading the page.
-AUTOSTART="FALSE" - tells the computer not to start the Video playing upon loading the page. You must click the start button to make the Video play.
-ALIGN=Middle - tells the computer to put the start/stop buttons to the middle.
-WIDTH= and HEIGHT= - are the dimensions of a small button panel that will appear when the page loads and contains both a START & STOP button so the visitor can start/stop the Video.
-LOOP=2 - will play the Video for two complete loops.
+src - tells what file to use.
+autostart="true" - tells the computer to start the Video playing upon loading the page.
+autostart="false" - tells the computer not to start the Video playing upon loading the page. You must click the start button to make the Video play.
+align=middle - tells the computer to put the start/stop buttons to the middle.
+width= and height= - are the dimensions of a small button panel that will appear when the page loads and contains both a START & STOP button so the visitor can start/stop the Video.
+loop=2 - will play the Video for two complete loops.
 -->
 """ % vars()
     return text
@@ -906,16 +908,16 @@ LOOP=2 - will play the Video for two complete loops.
 def html_movie_embed_qt(moviefile, width=400, height=400):
     """Return HTML for embedding a moviefile (QuickTime code)."""
     text = """
-<OBJECT CLASSID="clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B"
-CODEBASE="http://www.apple.com/qtactivex/qtplugin.cab"
-WIDTH="%(width)s" HEIGHT="%(height)s" >
-<PARAM NAME="src" VALUE="%(moviefile)s" >
-<PARAM NAME="autoplay" VALUE="true" >
-<EMBED SRC="%(moviefile)s"
-PLUGINSPAGE="http://www.apple.com/quicktime/download"
-WIDTH="%(width)s" HEIGHT="%(height)s" AUTOPLAY="true">
-</EMBED>
-</OBJECT>
+<object classid="clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B"
+codebase="http://www.apple.com/qtactivex/qtplugin.cab"
+width="%(width)s" height="%(height)s" >
+<param name="src" value="%(moviefile)s" >
+<param name="autoplay" value="false" >
+<embed src="%(moviefile)s"
+pluginspage="http://www.apple.com/quicktime/download"
+width="%(width)s" height="%(height)s" autoplay="false">
+</embed>
+</object>
 """ % vars()
     return text
 
@@ -925,7 +927,7 @@ def _test(d):
     # d is formatclass() or DocWriter(HTML, LaTeX, ...)
     print '\n\n', '*'*70, \
           '\n*** Testing class "%s"\n' % d.__class__.__name__, '*'*70
-    
+
     d.title('My Test of Class %s' % d.__class__.__name__,
             [('Hans Petter Langtangen',
               'Simula Research Laboratory',
@@ -995,7 +997,7 @@ b.item = 0  # create a new attribute
     d.table([['a', 'b'], ['c', 'd'], ['e', 'and a longer text']])
     print d
     d.write_to_file('tmp_%s' % d.__class__.__name__)
-        
+
 if __name__ == '__main__':
     formats = HTML, Doconce
     for format in formats:
